#   pdf-segmented: Generate PDFs using separate compression for foreground and background
#   Copyright (C) 2025  Lee Yingtong Li
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.

from . import CompressedLayer

from PIL import Image

from dataclasses import dataclass
import io
from typing import Optional

@dataclass
class JP2Layer(CompressedLayer):
	data: bytes

def jp2_compress_layer(layer: Image, jp2_lossless: bool, jp2_rate: Optional[float]) -> JP2Layer:
	save_kwargs = {}
	if jp2_rate is not None:
		save_kwargs['quality_mode'] = 'rates'
		save_kwargs['quality_layers'] = [jp2_rate]
	
	# Save image to JPEG 2000
	bytesio = io.BytesIO()
	layer.save(bytesio, format='jpeg2000', no_jp2=False, irreversible=not jp2_lossless, **save_kwargs)
	
	return JP2Layer(data=bytesio.getvalue())
